"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionFiles = void 0;
const Events_1 = require("../../common/Events");
const IdGenerator_1 = require("../../common/IdGenerator");
const SESSION_MARKER_PREFIX = 'bt-session';
const isDefined = (t) => !!t;
class SessionFiles {
    constructor(_fileSystem, _directory, sessionId, _maxPreviousLockedSessions = 1, _timestamp = Date.now(), _lockable = true) {
        this._fileSystem = _fileSystem;
        this._directory = _directory;
        this.sessionId = sessionId;
        this._maxPreviousLockedSessions = _maxPreviousLockedSessions;
        this._timestamp = _timestamp;
        this._lockable = _lockable;
        this._events = new Events_1.Events();
        this._locks = new Set();
        this._cleared = false;
        this._escapedSessionId = this.escapeFileName(sessionId);
        this.marker = this.getFileName(SESSION_MARKER_PREFIX);
    }
    initialize() {
        this.createSessionMarker();
    }
    getPreviousSession() {
        if (this._previousSession) {
            return this._previousSession;
        }
        const directoryFiles = this.readDirectoryFiles();
        if (!directoryFiles.length) {
            return undefined;
        }
        const sessionMarkers = directoryFiles
            .filter((f) => f.startsWith(SESSION_MARKER_PREFIX))
            .map((f) => this.getFileSession(f))
            .filter(isDefined)
            .sort((a, b) => b.timestamp - a.timestamp);
        const currentSessionMarker = sessionMarkers.find((s) => s.sessionId === this.sessionId);
        const lastSessionMarker = currentSessionMarker
            ? sessionMarkers.filter(({ timestamp }) => currentSessionMarker.timestamp > timestamp)[0]
            : sessionMarkers[0];
        if (!lastSessionMarker) {
            return undefined;
        }
        return (this._previousSession = new SessionFiles(this._fileSystem, this._directory, lastSessionMarker.sessionId, this._maxPreviousLockedSessions - 1, lastSessionMarker.timestamp, this._maxPreviousLockedSessions > 0));
    }
    getSessionWithId(sessionId) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        let session = this;
        while (session && session.sessionId !== sessionId) {
            session = session.getPreviousSession();
        }
        return session;
    }
    getPreviousSessions(count = Infinity) {
        const result = [];
        let current = this.getPreviousSession();
        while (current && count > 0) {
            result.push(current);
            count--;
            current = current.getPreviousSession();
        }
        return result;
    }
    lockPreviousSessions(lockId) {
        lockId = lockId !== null && lockId !== void 0 ? lockId : IdGenerator_1.IdGenerator.uuid();
        for (const session of this.getPreviousSessions(this._maxPreviousLockedSessions)) {
            session.lock(lockId);
        }
        return lockId;
    }
    unlockPreviousSessions(lockId) {
        for (const session of this.getPreviousSessions(this._maxPreviousLockedSessions)) {
            session.unlock(lockId);
        }
    }
    clearPreviousSessions() {
        for (const session of this.getPreviousSessions()) {
            session.clearSession();
        }
    }
    getFileName(prefix) {
        this.throwIfCleared();
        return this._directory + '/' + `${this.escapeFileName(prefix)}_${this._escapedSessionId}_${this._timestamp}`;
    }
    getSessionFiles() {
        this.throwIfCleared();
        const files = this.readDirectoryFiles();
        return files
            .map((file) => this.getFileSession(file))
            .filter(isDefined)
            .filter(({ sessionId }) => sessionId === this.sessionId)
            .map(({ file }) => this._directory + '/' + file);
    }
    clearSession() {
        if (this._locks.size > 0) {
            this._events.once('unlocked', () => this.clearSession());
            return;
        }
        if (this._cleared) {
            return;
        }
        try {
            const sessionFiles = this.getSessionFiles();
            for (const file of sessionFiles) {
                this._fileSystem.unlinkSync(file);
            }
        }
        catch (_a) {
            // Don't propagate errors
        }
        finally {
            this._cleared = true;
        }
    }
    lock(lockId) {
        if (this._cleared || !this._lockable) {
            return;
        }
        lockId = lockId !== null && lockId !== void 0 ? lockId : IdGenerator_1.IdGenerator.uuid();
        this._locks.add(lockId);
        return lockId;
    }
    unlock(lockId) {
        this._locks.delete(lockId);
        if (this._locks.size === 0) {
            this._events.emit('unlocked');
        }
    }
    getFileSession(file) {
        const [escapedSessionId, rawTimestamp] = this.splitByOneChar(file, '_').slice(-2);
        const timestamp = parseInt(rawTimestamp);
        if (isNaN(timestamp)) {
            return undefined;
        }
        return { file, escapedSessionId, timestamp, sessionId: this.unescapeFileName(escapedSessionId) };
    }
    readDirectoryFiles() {
        try {
            return this._fileSystem.readDirSync(this._directory);
        }
        catch (_a) {
            return [];
        }
    }
    createSessionMarker() {
        this._fileSystem.writeFileSync(this.marker, '');
    }
    escapeFileName(name) {
        return name.replace(/_/g, '__');
    }
    unescapeFileName(name) {
        return name.replace(/__/g, '_');
    }
    splitByOneChar(str, char) {
        const result = [];
        let start = 0;
        let index = str.indexOf(char);
        while (index !== -1) {
            if (str[index + 1] === char) {
                index = str.indexOf(char, index + 2);
            }
            else {
                result.push(str.substring(start, index));
                start = index + 1;
                index = str.indexOf(char, start);
            }
        }
        result.push(str.substring(start));
        return result;
    }
    throwIfCleared() {
        if (this._cleared) {
            throw new Error('This session files are cleared.');
        }
    }
}
exports.SessionFiles = SessionFiles;
//# sourceMappingURL=SessionFiles.js.map